// Join Reasonably 2

// joins the paths in the selection together with a reasonable order

// Preparation:
// 1. Make sure that all the end points of paths you want to connect
//    are closer than conf.threshold. If needed, move the anchor point
//    or draw a line.
// 2. Remove branched lines. It brings unexpected result.
//
// Specification:
// 1. Overlapped lines and isolated points are removed automatically
//    during the process.
// 2. This scripts joins the anchor points. Any condition of handles
//    are ignored.
// 3. The style of the frontmost path in the selection is applied
//    for all the resulted paths.

var conf ={};
// Settings ==================================

conf.close  = true;     // makes close paths. true/false

conf.threshold   = 0.1;  // merges the ends to connect within this distance (unit:mm)

conf.alert_when_done = true; // shows alert dialog at the end. true/false

// ========================================
// JavaScript Script for Adobe Illustrator CS5
// Tested with Adobe Illustrator CS5 15.0.2, Windows XP SP3 (Japanese version).
// This script provided "as is" without warranty of any kind.
// Free to use and distribute.

// Copyright(c) 2012 Hiroyuki Sato
// http://park12.wakwak.com/~shp/lc/et/en_aics_script.html

// 2012-04-29 release


function mm2pt(mm){  return mm * 2.83464567;  }
conf.threshold = mm2pt( conf.threshold );
conf.t2 = conf.threshold * conf.threshold;

// ------------------------------------------------
function SegmentArray(){
  this.paths = [];
  this.segs = [];
  
  this.add = function(p){ // p = PathItem
    var pp = p.pathPoints;

    this.paths.push( p );

    var i, j;
    var exist;
    
    for(i=0; i<pp.length; i++){
      if(i == pp.length - 1) break;

      if( dist2( pp[i].anchor, pp[i+1].anchor ) <= conf.t2 ) continue;

      exist = false;
      
      for(j=0; j<this.segs.length; j++){
        if( (dist2( pp[i].anchor, this.segs[j][0] ) <= conf.t2 
             && dist2( pp[i+1].anchor, this.segs[j][1] ) <= conf.t2 )
            || (dist2( pp[i+1].anchor, this.segs[j][0] ) <= conf.t2 
                && dist2( pp[i].anchor, this.segs[j][1] ) <= conf.t2 )){
          exist = true;
          break;
        }
      }
      if( ! exist ){
        this.segs.push( [pp[i].anchor, pp[i+1].anchor] );
      }
    }
  }
  
  this.getNearest = function(pnt){ // pnt = [x, y]
    var D = {d:null, idx:null, xy:null};
    var d0 = null;
    var d1 = null;
    
    for(var i = 0; i < this.segs.length; i++){
      d0 = dist2( this.segs[i][0], pnt );
      d1 = dist2( this.segs[i][1], pnt );
      
      if(D.d == null){
        D.idx = i;
        if( d0 < d1 ){
          D.d = d0;
          D.xy = 0;
        } else {
          D.d = d1;
          D.xy = 1;
        }
      } else {
        if( D.d > d0 ){
          D.d = d0;
          D.idx = i;
          D.xy = 0;
        }
        if( D.d > d1 ){
          D.d = d1;
          D.idx = i;
          D.xy = 1;
        }
      }
    }
    return D;
  }
}
// ------------------------------------------------
main();
function main(){
  var s = [];
  getPathItemsInSelection(0, s);
  if(s.length < 2) return; // it needs at least 2 paths

  var sr = new SegmentArray();
  var i;
  for(i = 0; i < s.length; i++){
    if(s[i].pathPoints.length == 1) continue;
    sr.add( s[i] );
  }
  
  if( sr.paths.length < 2) return;
  createPathFromSegments( sr );
  
  for(i=0; i<s.length; i++ ){
    s[i].remove();
  }

  if( conf.alert_when_done ){
    alert("done");
  }
}
// ------------------------------------------------
function createPathFromSegments( sr ){
  var D0, D1;
  var i, seg, pnts, path;
  
  while( sr.segs.length > 0 ){
    seg = sr.segs[0];
    pnts = [ seg[0], seg[1] ];
    sr.segs.shift();

    while( sr.segs.length > 0 ){
      D0 = sr.getNearest( pnts[0] );
      D1 = sr.getNearest( pnts[pnts.length - 1] );
    
      if( D0.d == null || D1.d == null){
        alert("caution: something unexpected occurs during comparing the distances");
        break;
      }
      if(Math.min(D0.d, D1.d) >= conf.t2) break;
      
      if( D0.d < D1.d ){
        if( D0.xy == 0 ){
          if(D0.d > conf.t2) pnts.unshift( sr.segs[ D0.idx ][0] );
          pnts.unshift( sr.segs[ D0.idx ][1] );
        } else {
          if(D0.d > conf.t2) pnts.unshift( sr.segs[ D0.idx ][1] );
          pnts.unshift( sr.segs[ D0.idx ][0] );
        }
        sr.segs.splice( D0.idx,1 );
      } else {
        if( D1.xy == 0 ){
          if(D1.d > conf.t2) pnts.push( sr.segs[ D1.idx ][0] );
          pnts.push( sr.segs[ D1.idx ][1] );
        } else {
          if(D1.d > conf.t2) pnts.push( sr.segs[ D1.idx ][1] );
          pnts.push( sr.segs[ D1.idx ][0] );
        }
        sr.segs.splice( D1.idx,1 );
      }
    }

    if( pnts.length > 1 ){
      if(dist2( pnts[0], pnts[pnts.length - 1]) <= conf.t2 ){
        pnts.pop();
      }
    }

    if( pnts.length > 1 ){
      path = sr.paths[0].duplicate(sr.paths[0].layer, ElementPlacement.PLACEATBEGINNING );
      for(i=path.pathPoints.length - 1; i>0 ; i--){
        path.pathPoints[i].remove();
      }
      with( path.pathPoints[0] ){
        anchor = pnts[0];
        rightDirection = anchor;
        leftDirection = anchor;
      }
      for(i=1; i<pnts.length; i++){
        with( path.pathPoints.add() ){
          anchor = pnts[i];
          rightDirection = anchor;
          leftDirection = anchor;
        }
      }
      if(conf.close && pnts.length > 2) path.closed = true;
    }
  }
}
// ------------------------------------------------
// return square of distance
function dist2(p1, p2) {
  return Math.pow(p1[0] - p2[0],2) + Math.pow(p1[1] - p2[1],2);
}
// ------------------------------------------------
// extract PathItems from the selection which length of PathPoints
// is greater than "n"
function getPathItemsInSelection(n, paths){
  if(documents.length < 1) return;
  
  var s = activeDocument.selection;
  
  if (!(s instanceof Array) || s.length < 1) return;

  extractPaths(s, n, paths);
}

// --------------------------------------
// extract PathItems from "s" (Array of PageItems -- ex. selection),
// and put them into an Array "paths".  If "pp_length_limit" is specified,
// this function extracts PathItems which PathPoints length is greater
// than this number.
function extractPaths(s, pp_length_limit, paths){
  for(var i = 0; i < s.length; i++){
    if(s[i].typename == "PathItem"
       && !s[i].guides && !s[i].clipping){
      if(pp_length_limit
         && s[i].pathPoints.length <= pp_length_limit){
        continue;
      }
      paths.push(s[i]);
      
    } else if(s[i].typename == "GroupItem"){
      // search for PathItems in GroupItem, recursively
      extractPaths(s[i].pageItems, pp_length_limit, paths);
      
    } else if(s[i].typename == "CompoundPathItem"){
      // searches for pathitems in CompoundPathItem, recursively
      // ( ### Grouped PathItems in CompoundPathItem are ignored ### )
      extractPaths(s[i].pathItems, pp_length_limit , paths);
    }
  }
}
